@@ -21,6 +21,7 @@ from typing import Callable, Optional, Set, TextIO, Union
 
 PREFIX = "SystemLog:"
 SCRUB_MESSAGE = "**Exception message scrubbed**"
+LATEST_EXCEPTION = None
 
 
 class PublicValueError(ValueError):
@@ -335,14 +336,15 @@ def print_prefixed_stack_trace_and_raise(
             empty all messages will be srubbed.
         err: the error that was thrown. None accepted for backwards compatibility.
     """
+    global LATEST_EXCEPTION
     if err is None:
         err = sys.exc_info()[1]
     scrubbed_err = err
-    if err not in SCRUBBED_EXCEPTIONS:
+    if err != LATEST_EXCEPTION:
         scrubbed_err = scrub_exception(
             err, scrub_message, prefix, keep_message, allow_list
         )
-        SCRUBBED_EXCEPTIONS.add(scrubbed_err)
+        LATEST_EXCEPTION = err
 
     tb_exception = TracebackException.from_exception(scrubbed_err)  # type: ignore
 
@@ -434,8 +436,6 @@ def prefix_stack_trace(
         def foo(x):
             pass
     """
-    global SCRUBBED_EXCEPTIONS
-    SCRUBBED_EXCEPTIONS = set()
     return _PrefixStackTraceWrapper(
         file, disable, prefix, scrub_message, keep_message, allow_list, add_timestamp
     )
@@ -459,8 +459,6 @@ class PrefixStackTrace:
         self.keep_message = keep_message
         self.add_timestamp = add_timestamp
         self.allow_list = allow_list
-        global SCRUBBED_EXCEPTIONS
-        SCRUBBED_EXCEPTIONS = set()
 
     def __enter__(self):
         pass
