@@ -331,23 +331,30 @@ class AMLModuleLoader:
                         module_name, module_version, module_cache_key
                     )
                 except BaseException as e:
-                    # save the exception to raise it if namespace not provided
-                    if not module_namespace:
-                        raise e
+                    log.debug(f"Exception: {e}")
 
-                log.info(
-                    f"    Trying to load module {module_name} with namespace {module_namespace}."
-                )
-                module_name = module_namespace + "://" + module_name
-                return self.solve_module_version_and_load(
-                    module_name, module_version, module_cache_key
-                )
+                if module_namespace is not None:
+                    log.info(
+                        f"    Trying to load module {module_name} with namespace {module_namespace}."
+                    )
+                    module_name = module_namespace + "://" + module_name
+
+                    try:
+                        return self.solve_module_version_and_load(
+                            module_name, module_version, module_cache_key
+                        )
+                    except BaseException as e:
+                        log.debug(f"Exception: {e}")
 
             else:
                 raise ValueError(
-                    f"Loading order contains incorrect key {i}. Please use a comma-separated str to order 'registry', 'feed', and 'workspace'."
+                    f"Loading order contains incorrect key {source}. Please use a comma-separated str to order 'registry', 'feed', and 'workspace'."
                 )
 
+        raise ValueError(
+            f"Module {module_name} with version {module_version} not found in neither of the following sources: {loading_order}."
+        )
+
     def load_prod_module(
         self, module_name, module_version, from_workspace=False, module_namespace=None
     ):
