@@ -16,7 +16,7 @@ import uuid
 import shutil
 import yaml
 import jsonpath_ng
-from typing import Callable
+from typing import Callable, Optional
 from toposort import toposort_flatten
 import datetime
 from dataclasses import dataclass
@@ -36,6 +36,7 @@ try:
     from azureml.core import Dataset
     from azureml.pipeline.core import PipelineRun
     from azure.ml.component.component import Component, Input, Output
+    from azure.ml.component.pipeline import Pipeline
     from azureml.exceptions._azureml_exception import UserErrorException
 except ImportError as error:
     raise ImportError(
@@ -417,6 +418,14 @@ class AMLPipelineHelper:
                 f"Configured output {output_key} to use mode {output_mode} and datastore {datastore_name}"
             )
 
+    def _get_dc_datastore(self):
+        if self.config.compute.dc_datastore:
+            return self.config.compute.dc_datastore
+        else:
+            raise UserErrorException(
+                "Please specify `compute.dc_datastore` in your compute yaml."
+            )
+
     def _apply_windows_runsettings(
         self,
         module_name,
@@ -448,8 +457,11 @@ class AMLPipelineHelper:
                 if target is not None
                 else self.config.compute.windows_cpu_dc_target
             )
-            if self.config.compute.hdi_dc_target:
-                datastore_name = self.config.compute.dc_datastore
+            if (
+                self.config.compute.hdi_dc_target
+                or self.config.compute.synapse_dc_target
+            ):
+                datastore_name = self._get_dc_datastore()
         else:
             target = (
                 target
@@ -533,7 +545,7 @@ class AMLPipelineHelper:
                 raise Exception(
                     f"Your HDI component {module_name} is using local version. Please specify hdi_dc_target"
                 )
-            datastore_name = self.config.compute.dc_datastore
+            datastore_name = self._get_dc_datastore()
         else:
             target = (
                 target if target is not None else self.config.compute.hdi_prod_target
@@ -622,14 +634,17 @@ class AMLPipelineHelper:
             custom_runtime_arguments (dict): any additional custom args
         """
 
-        if self.module_loader.is_local(module_name) and self.is_eyesoff():
-            log.warning(
-                "The job is expected to fail, as local Synapse components are not supported in eyes-off environments"
+        if self.module_loader.is_local(module_name):
+            target = (
+                target if target is not None else self.config.compute.synapse_dc_target
+            )
+            datastore_name = self._get_dc_datastore()
+        else:
+            target = (
+                target
+                if target is not None
+                else self.config.compute.synapse_prod_target
             )
-
-        target = (
-            target if target is not None else self.config.compute.synapse_prod_target
-        )
         log.info(
             f"Using Synapse compute target {target} to run {module_name} from pipeline class {self.__class__.__name__}"
         )
@@ -653,6 +668,7 @@ class AMLPipelineHelper:
                     f"compute.synapse_conf of type {type(new_conf)} is not a valid json string or a single tested configuration."
                 )
             if self.module_loader.is_local(module_name):
+                # https://msdata.visualstudio.com/Vienna/_workitems/edit/1918995
                 new_conf = json.dumps(new_conf)
             module_instance.runsettings.spark.configure(
                 conf=new_conf,
@@ -750,8 +766,11 @@ class AMLPipelineHelper:
                     _target = self.config.compute.linux_gpu_dc_target
                 else:
                     _target = self.config.compute.linux_cpu_dc_target
-            if self.config.compute.hdi_dc_target:
-                datastore_name = self.config.compute.dc_datastore
+            if (
+                self.config.compute.hdi_dc_target
+                or self.config.compute.synapse_dc_target
+            ):
+                datastore_name = self._get_dc_datastore()
         else:
             if windows:
                 if gpu:
@@ -844,8 +863,11 @@ class AMLPipelineHelper:
                 if target is not None
                 else self.config.compute.linux_gpu_dc_target
             )
-            if self.config.compute.hdi_dc_target:
-                datastore_name = self.config.compute.dc_datastore
+            if (
+                self.config.compute.hdi_dc_target
+                or self.config.compute.synapse_dc_target
+            ):
+                datastore_name = self._get_dc_datastore()
             log.info(
                 f"Using target {target} for local code GPU module {module_name} from pipeline class {self.__class__.__name__}"
             )
@@ -864,8 +886,11 @@ class AMLPipelineHelper:
                 if target is not None
                 else self.config.compute.linux_cpu_dc_target
             )
-            if self.config.compute.hdi_dc_target:
-                datastore_name = self.config.compute.dc_datastore
+            if (
+                self.config.compute.hdi_dc_target
+                or self.config.compute.synapse_dc_target
+            ):
+                datastore_name = self._get_dc_datastore()
             log.info(
                 f"Using target {target} for local CPU module {module_name} from pipeline class {self.__class__.__name__}"
             )
@@ -1063,8 +1088,11 @@ class AMLPipelineHelper:
                     target = self.config.compute.linux_gpu_dc_target
                 else:
                     target = self.config.compute.linux_cpu_dc_target
-            if self.config.compute.hdi_dc_target:
-                datastore_name = self.config.compute.dc_datastore
+            if (
+                self.config.compute.hdi_dc_target
+                or self.config.compute.synapse_dc_target
+            ):
+                datastore_name = self._get_dc_datastore()
         else:
             if windows:
                 if gpu:
@@ -1713,9 +1741,11 @@ class AMLPipelineHelper:
             except BaseException as e:
                 log.error(f"An error occurred, recovery is not successful: {e}")
 
-    def _update_hdi_dc_configs(self, pipeline):
-        if not self.config.compute.hdi_dc_target:
-            log.info("This pipeline doesn't involve HDI DC.")
+    def _update_dc_configs(self, pipeline):
+        hdi_compute = self.config.compute.hdi_dc_target
+        synapse_compute = self.config.compute.synapse_dc_target
+        if not hdi_compute and not synapse_compute:
+            log.info("This pipeline doesn't involve HDI DC or Synapse DC.")
             return pipeline
         nodes = pipeline._expand_pipeline_nodes()
         graph = defaultdict(list)
@@ -1740,24 +1770,24 @@ class AMLPipelineHelper:
             node_mapping[n._instance_id] = n
             graph[n._instance_id].append(n.nodes[-1]._instance_id)
         sorted_node_ids = toposort_flatten(graph)
-        use_hdi_dc = defaultdict(bool)
+        use_dc = defaultdict(bool)
         for id in sorted_node_ids:
             if id not in node_mapping:
                 continue
             node = node_mapping[id]
-            if (
-                node.type != "ScopeComponent"
-                and node.runsettings.target == self.config.compute.hdi_dc_target
+            if node.type != "ScopeComponent" and (
+                (hdi_compute and node.runsettings.target == hdi_compute)
+                or (synapse_compute and node.runsettings.target == synapse_compute)
             ):
-                use_hdi_dc[id] = True
+                use_dc[id] = True
                 continue
             for parent in graph[id]:
-                if use_hdi_dc[parent]:
-                    use_hdi_dc[id] = True
+                if use_dc[parent]:
+                    use_dc[id] = True
                     break
 
         for id, node in node_mapping.items():
-            if use_hdi_dc[id]:
+            if use_dc[id]:
                 if node.type == "ScopeComponent":
                     # scope component doesn't need to configure compute target
                     # output datastore has to be cosmos (ADLSg1), while dc_datastore is ADLSg2
@@ -1791,6 +1821,10 @@ class AMLPipelineHelper:
                     node.runsettings.configure(
                         target=self.config.compute.windows_cpu_dc_target
                     )
+                elif node.runsettings.target == self.config.compute.synapse_prod_target:
+                    node.runsettings.configure(
+                        target=self.config.compute.synapse_dc_target
+                    )
                 else:
                     continue
                 log.info(
@@ -1849,15 +1883,17 @@ class AMLPipelineHelper:
                 aml_force=self.config.aml.force,
             )  # NOTE: this also stores aml workspace in internal global variable
 
-    def build_and_submit_new_pipeline(self):
+    def _build_pipeline(self) -> Pipeline:
         log.info(f"Building Pipeline [{self.__class__.__name__}]...")
         pipeline_function = self.build(self.config)
 
         log.info("Creating Pipeline Instance...")
         pipeline = self.pipeline_instance(pipeline_function, self.config)
 
-        log.info("Overriding compute target if upstream steps use HDI DC...")
-        pipeline = self._update_hdi_dc_configs(pipeline)
+        log.info(
+            "Overriding compute target if upstream steps use HDI DC or Synapse DC..."
+        )
+        pipeline = self._update_dc_configs(pipeline)
 
         if not self.config.run.skip_validation:
             log.info("Validating...")
@@ -1867,7 +1903,14 @@ class AMLPipelineHelper:
             log.info(f"Exporting to {self.config.run.export}...")
             with open(self.config.run.export, "w") as export_file:
                 export_file.write(pipeline._get_graph_json(workspace=self.workspace()))
+        return pipeline
 
+    def _submit_pipeline(self, pipeline: Pipeline) -> Optional[PipelineRun]:
+        """Publish and submit the given pipeline.
+
+        Args:
+            pipeline: pipeline to be submitted
+        """
         if self.config.run.publish:
             self._publish_to_endpoint(pipeline)
 
@@ -1908,8 +1951,19 @@ class AMLPipelineHelper:
             )
             return
 
-    def run(self):
-        """Run pipeline using arguments"""
+    def build_and_submit_new_pipeline(self) -> Optional[PipelineRun]:
+        """Build the pipeline and submit it."""
+        pipeline = self._build_pipeline()
+        return self._submit_pipeline(pipeline)
+
+    def run(self, pipeline: Optional[Pipeline] = None) -> None:
+        """Run pipeline using arguments.
+
+        Args:
+            pipeline: If set to `None`, the pipeline helper will take care of
+              building a pipeline via it's `build` functions, otherwise the
+              pipeline passed here will be used.
+        """
         # set logging level
         if self.config.run.verbose:
             logging.getLogger().setLevel(logging.DEBUG)
@@ -1952,6 +2006,11 @@ class AMLPipelineHelper:
 
         pipeline_run = None
         if self.config.run.resume:
+            if pipeline is not None:
+                raise Exception(
+                    "Did not expect a custom pipeline instance to be given when resuming a pipeline run ."
+                    "Either pass `pipeline=None` or set `self.config.run.resume=False`."
+                )
             if not self.config.run.pipeline_run_id:
                 raise Exception(
                     "To be able to use --resume you need to provide both --experiment-name and --run-id."
@@ -1983,7 +2042,10 @@ class AMLPipelineHelper:
                         log.error(f"An error occurred, override is not successful: {e}")
                         raise e
             try:
-                pipeline_run = self.build_and_submit_new_pipeline()
+                if pipeline is None:
+                    pipeline_run = self.build_and_submit_new_pipeline()
+                else:
+                    pipeline_run = self._submit_pipeline(pipeline=pipeline)
             except BaseException as e:
                 log.error(f"An error {e} occurred during pipeline submission.")
                 if override:
